/* --------------------------------------------------------------- */
/* Licensed Materials - Property of IBM                            */
/* 5724-S84                                                        */
/* (C) Copyright IBM Corp. 2007,2008, All Rights Reserved          */
/* US Government Users Restricted Rights - Use, duplication or     */
/* disclosure restricted by GSA ADP Schedule Contract with         */
/* IBM Corp.                                                       */
/* --------------------------------------------------------------- */
/* PROLOG END TAG zYx                                              */
/*

   FFT1D_sample.c - a simple routine to drive the SPU functions in the fft library

*/


#include <stdio.h>
#include <sched.h>
#include <stdlib.h>
#include <strings.h>
#include <time.h>
#include <sys/time.h>
#include <libspe2.h>

#include <libfft.h>

#define IFFT_TYPE_C2C (FFT_TYPE_C2C + 1)

// The sample SPU program that actually does the work.
extern spe_program_handle_t FFT1D_sample_spu;

// Forward declare.
void test_1D(int sizeOfFfts, int flavor);


// Main.
int main(int argc, char *argv[])
{
  if (argc != 3)
  {
    fprintf(stderr, "Usage: %s <function type = c2c, ic2c, r2c, c2r> <size of each fft>\n", argv[0]);
    exit(1);
  }

  int sizeOfFfts = atoi(argv[2]); 

  int flavor;
  if (strcasecmp(argv[1], "c2c") == 0)
  {
    flavor = FFT_TYPE_C2C;
  }
  else if (strcasecmp(argv[1], "ic2c") == 0)
  {
    flavor = IFFT_TYPE_C2C;
  }
  else if (strcasecmp(argv[1], "r2c") == 0)
  {
    flavor = FFT_TYPE_R2C;
  }
  else if (strcasecmp(argv[1], "c2r") == 0)
  {
    flavor = FFT_TYPE_C2R;
  }
  else
  {
    fprintf(stderr, "Bad function type.\n");
    exit(1);
  }

  test_1D(sizeOfFfts, flavor);

  return 0;
} // end main




// Launch the SPU thread to perform an FFT.
// This sample launches one SPU thread which then performs a single FFT.
void test_1D(int sizeOfFfts, int flavor)
{
  // Start timer.
  struct timeval start, end;
  gettimeofday(&start, NULL);

  // Load the code on to an SPU.
  spe_program_handle_t *code = &FFT1D_sample_spu;
  spe_context_ptr_t speContext = spe_context_create(0, NULL);
  spe_program_load(speContext, code);
  // Pass the flavor and numberOfFfts to the SPU code.
  unsigned int entry = SPE_DEFAULT_ENTRY;
  spe_context_run(speContext, &entry, 0, (void *)flavor, (void *)sizeOfFfts, NULL);
  spe_context_destroy(speContext);

  // Stop timer.
  gettimeofday(&end, NULL);
  unsigned int elapsed = ((end.tv_sec * 1000000) + end.tv_usec) - ((start.tv_sec * 1000000) + start.tv_usec);
  fprintf(stderr, "Calculation time took %u usec.\n", elapsed);
}

