/* --------------------------------------------------------------- */
/* Licensed Materials - Property of IBM                            */
/* 5724-S84                                                        */
/* (C) Copyright IBM Corp. 2007,2008, All Rights Reserved          */
/* US Government Users Restricted Rights - Use, duplication or     */
/* disclosure restricted by GSA ADP Schedule Contract with         */
/* IBM Corp.                                                       */
/* --------------------------------------------------------------- */
/* PROLOG END TAG zYx                                              */

/*

   FFT1D_sample_spu.c - a simple routine to drive the SPU functions in the fft library

*/


#include <spu_mfcio.h>

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include <libfft_spu.h>

#define IFFT_TYPE_C2C (FFT_TYPE_C2C + 1)

// Forward declares.
void generateC2Cdata(int sizeOfFfts, float *problems);
void generateR2Cdata(int sizeOfFfts, float *problems);
void generateC2Rdata(int sizeOfFfts, float *problems);

// Input and output data.
float input_data[20480] __attribute__ ((aligned (128)));
float output_data[20480] __attribute__ ((aligned (128)));


int main(unsigned long long spuid, unsigned long long argp, unsigned long long envp ) {

  // The number of FFTs we do in this sample is always one at a time.
  unsigned int flavor = (unsigned int)argp;
  unsigned int sizeOfFfts = (unsigned int)envp;

  int j;

  if (sizeOfFfts*2 > 20480)
  {
    fprintf(stderr, "This sample only supports FFT sizes up to 10240.\n");
    return 1;
  }

  // Populate input data and perform FFT.
//srand(spuid);
  srand(1);

  int res = -1;
  if (flavor == FFT_TYPE_C2C)
  {
    generateC2Cdata(sizeOfFfts, input_data);
    res = fft_1d_c2c_spu(sizeOfFfts, (vector float*)input_data, (vector float*)output_data, 0);
  }
  else if (flavor == IFFT_TYPE_C2C)
  {
    generateC2Cdata(sizeOfFfts, input_data);
    res = fft_1d_c2c_spu(sizeOfFfts, (vector float*)input_data, (vector float*)output_data, 1);
  }
  else if (flavor == FFT_TYPE_R2C)
  {
    generateR2Cdata(sizeOfFfts, input_data);
    res = fft_1d_r2c_spu(sizeOfFfts, (vector float*)input_data, (vector float*)output_data, 0);
  }
  else if (flavor == FFT_TYPE_C2R)
  {
    generateC2Rdata(sizeOfFfts, input_data);
    res = fft_1d_c2r_spu(sizeOfFfts, (vector float*)input_data, (vector float*)output_data, 0);
  }

  if (res == FFT_RC_SUCCESS)
  {
    fprintf(stderr, "SPU %llx completed FFT successfully.\n", spuid);
#if 0
    for (j = 0; j < (int)sizeOfFfts; j++) {
      printf("X_%d: %.4f + %.4fi\n", j, output_data[j*2], output_data[j*2+1]);
      }
#endif
  }
  else
  {
    fprintf(stderr, "SPU %llx completed FFT with return code %d.\n", spuid, res);
  }

  return 0;
}


// Generate complex numbers as input.
void generateC2Cdata(int sizeOfFfts, float *problems)
{
  int j;
  for (j=0; j<sizeOfFfts; ++j)
  {
    problems[j*2] = rand() % 1024; // Real
    problems[j*2+1] = rand() % 1024; // Imag
  }
}

// Generate packed reals as input.
void generateR2Cdata(int sizeOfFfts, float *problems)
{
  int j;
  for (j=0; j<sizeOfFfts; ++j)
  {
    problems[j] = rand() % 1024; // Real
  }
  // Don't care about the elements in the other half of the array,
  // since they don't get used.
}

// Generate complex conjugates as input.
void generateC2Rdata(int sizeOfFfts, float *problems)
{
  problems[0] = rand() % 1024; // Real
  problems[1] = 0; // Imag
  int j;
  for (j=1; j<(sizeOfFfts+1)/2; ++j)
  {
    problems[j*2] = rand() % 1024; // Real
    problems[j*2+1] = rand() % 1024; // Imag
    // Complex conjugate.
    problems[(sizeOfFfts-j)*2] = problems[j*2]; // Real
    problems[(sizeOfFfts-j)*2+1] = -problems[j*2+1]; // Imag
  }
  if (!(sizeOfFfts % 2)) // Size is even.
  {
    problems[sizeOfFfts] = rand() % 1024; // Real
    problems[sizeOfFfts+1] = 0; // Imag
  }
}

