/* --------------------------------------------------------------- */
/* Licensed Materials - Property of IBM                            */
/* 5724-S84                                                        */
/* (C) Copyright IBM Corp. 2008       All Rights Reserved          */
/* US Government Users Restricted Rights - Use, duplication or     */
/* disclosure restricted by GSA ADP Schedule Contract with         */
/* IBM Corp.                                                       */
/* --------------------------------------------------------------- */
/* PROLOG END TAG zYx                                              */
/*
FFT2D_DP_sample.c - a simple routine to drive the fft library.
Sample for FFT2D double precision. 
*/
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <time.h>
#include <sys/time.h>
#include <unistd.h>

#include <libfft.h>

/*Forward declare.*/
void test_2D (int numCols, int numRows, int spusToUse);

/*Main*/
int main (int argc, char *argv[])
{
	if (argc != 4)
	{
		fprintf (stderr,
			"Usage: %s <spus to use> <number of FFT columns> <number of FFT rows>\n",
			argv[0]);
		exit (1);
	}

	int spusToUse = atoi (argv[1]);
	int numCols = atoi (argv[2]);
	int numRows = atoi (argv[3]);
	if (numCols < 1 || numCols > 2048)
	{
		fprintf (stderr, "Bad number of columns. Only 1-2048 are supported.\n");
		exit (1);
	}
	if (numRows < 1 || numRows > 2048)
	{
		fprintf (stderr, "Bad number of rows. Only 1-2048 are supported.\n");
		exit (1);
	}

	test_2D (numCols, numRows, spusToUse);

	return 0;
}/*end main*/


/*Generate complex numbers as input.*/
void generateC2Cdata (int numCols, int numRows, double *problems, double *output)
{
	int counter = 0;
	int i;
	for (i = 0; i < numRows; ++i)
	{
		int j;
		for (j = 0; j < numCols; ++j)
		{
			problems[counter] = rand () % 512;  /*Real*/
			output[counter++] = 0.0f;     /*Touch the output memory to avoid performance hit.*/
			problems[counter] = rand () % 512;  /*Imag*/
			output[counter++] = 0.0f;     /* Touch the output memory to avoid performance hit.*/
		}
	}
}

/* Allocate space for data and perform the FFT.*/
void test_2D (int numCols, int numRows, int spusToUse)
{

	/*Allocate storage for input and output data.*/
	void *ptr;
	int inverse_flag = 0;/*inverse_flag = 0 ; stand for forward FFT.*/
	int type;
	posix_memalign (&ptr, 128, sizeof (double) * 2 * numCols * numRows);
	double *input_data = (double *) ptr;
	posix_memalign (&ptr, 128, sizeof (double) * 2 * numCols * numRows);
	double *output_data = (double *) ptr;

	/*Populate input data.*/
	generateC2Cdata (numCols, numRows, (double *)input_data,(double *) output_data);

	/*Call library to process.*/
	fft_2d_handle_t handle;
	/*start timer*/
	struct timeval start, end;
	gettimeofday(&start,NULL);
	unsigned int elapsed;

	inverse_flag = 0;
	type = FFT_TYPE_C2C;
	/*Initialize the SPUs.*/
	int res = fft_2d_dp_initialize (&handle, spusToUse);
	if (res == FFT_RC_SUCCESS)
	{
		/*Perform the transform.*/

		res = fft_2d_dp_perform (handle, numCols, numRows, (void **) &input_data,(void **) &output_data, inverse_flag, type);

		if (res != FFT_RC_SUCCESS)
		{
			fprintf (stderr, "FFT failure: %d\n", res);
		}
		/*Terminate the FFT 2d for double precision*/
		fft_2d_dp_terminate (handle);
	}
	else
	{
		fprintf (stderr, "FFT failed to initialize: %d\n", res);
	}
	
	/*stop timer*/
	gettimeofday(&end,NULL);
	elapsed = (( end.tv_sec * 1000000) + end.tv_usec) - ((start.tv_sec * 1000000) + start.tv_usec) ;
	fprintf(stderr, "Calculation time took %u usec.\n",elapsed);



	/*Cleanup.*/
	free (input_data);
	free (output_data);
}
